##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: interfaces.py 73916 2007-03-29 15:28:25Z dobe $
"""
__docformat__ = "reStructuredText"

import zope.schema
import zope.schema.interfaces

from z3c.schema.i18n import MessageFactory as _


class IOptionalChoice(zope.schema.interfaces.IChoice,
                      zope.schema.interfaces.IFromUnicode):
    """Optional Choice

    This field can either represent a choice or a textline value.

    Validation proceeds as follows:

    1. Check whether the value is one of the choices. If so, return
       successfully.

    2. If no choice match was found, validate the value against the
       ``value_type`` field.

    The conversion from unicode values proceeds in a similar fashion:

    1. Try to match the unicode value to a token in the vocabulary.

    2. If no match was made, call the unicode conversion method of the
       ``value_type`` field.
    """

    value_type = zope.schema.Field(
        title = _("Value Type"),
        description = _(u"The freely entered values must be of this type."),
        required=True)



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.interface
import interfaces

def isValidCreditCard(cardNum):
    """Returns True if the credit card number is a valid Luhn (Mod 10) number
       and False if not. This, of course, does not validate the number, but 
       will catch typos. There is the chance that two typographic errors could
       return a false positive if they offset one anoter, but the likelihood
       is low and pre-validating is fast"""
    
    financialIndustries = ['3','4','5','6']
    if cardNum[1] not in financialIndustries:
        return False
        
    total = pos = 0
    for digit in cardNum[::-1]:
        if pos % 2 == 0:
            multiplier = 1
        else:
            multiplier = 2
        product = int(digit) * multiplier
        total += product / 10 + product % 10
        pos += 1
    if total % 10 == 0:
        return True
    return False
    
class ISO7812CreditCard(zope.schema.TextLine):
    """A valid ISO 7812 credit card number."""
    __doc__ = interfaces.IISO7812CreditCard.__doc__

    zope.interface.implements(interfaces.IISO7812CreditCard)

    def constraint(self, value):
        allDigits = True
        for char in value[:]:
            if not char.isdigit():
                allDigits = False
        return '\n' not in value and '\r' not in value and allDigits

    def _validate(self, value):
        super(ISO7812CreditCard, self)._validate(value)
        if not isValidCreditCard(value):
            raise interfaces.NotValidISO7812CreditCard(value)


##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: field.py 91149 2008-09-14 22:39:34Z rogerineichen $
"""
__docformat__ = "reStructuredText"

import re
import zope.interface
import zope.schema

from z3c.schema.regex import interfaces


class Regex(zope.schema.ASCIILine):
    """Regex schema field.

    Must be a compilable regular expression
    """

    zope.interface.implements(interfaces.IRegex)

    def _validate(self, value):
        super(Regex, self)._validate(value)
        try:
            re.compile(value)
        except re.error, e:
            raise interfaces.InvalidRegex, '%r, %s' % (value, e)

    def fromUnicode(self, value):
        v = str(value.strip())
        self.validate(v)
        return v



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: field.py 91149 2008-09-14 22:39:34Z rogerineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.schema
from zope.schema.interfaces import IField, ValidationError

from z3c.schema.optchoice import interfaces


class OptionalChoice(zope.schema.Choice):
    """Optional Choice field."""

    zope.interface.implements(interfaces.IOptionalChoice)

    def __init__(self, value_type, **kw):
        super(OptionalChoice, self).__init__(**kw)
        # whine if value_type is not a field
        if value_type is not None and not IField.providedBy(value_type):
            raise ValueError("'value_type' must be field instance.")
        self.value_type = value_type

    def _validate(self, value):
        try:
            super(OptionalChoice, self)._validate(value)
        except ValidationError:
            self.value_type._validate(value)

    def fromUnicode(self, value):
        try:
            return super(OptionalChoice, self).fromUnicode(value)
        except ValidationError:
            return self.value_type.fromUnicode(value)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: interfaces.py 92501 2008-10-23 08:40:27Z nadako $
"""
__docformat__ = "reStructuredText"

import zope.schema
import zope.schema.interfaces

from z3c.schema.i18n import MessageFactory as _


class IHostName(zope.schema.interfaces.IURI, 
                zope.schema.interfaces.IFromUnicode):
    """Host name field."""


class InvalidHostName(zope.schema.ValidationError):
    __doc__ = _("""The specified host name is not valid.""")



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: field.py 91149 2008-09-14 22:39:34Z rogerineichen $
"""
__docformat__ = "reStructuredText"

import re

import zope.interface
import zope.schema

from z3c.schema.baseurl import interfaces


isValidBaseURL = re.compile(
    r"[a-zA-z0-9+.-]+:"   # scheme
    r"\S*$"               # non space (should be pickier)
    ).match


class BaseURL(zope.schema.URI):
    """Base URL field.
    
    Such a base url must end with a ``/``. This makes it simpler for
    append a view name.
    """

    zope.interface.implements(interfaces.IBaseURL)

    def _validate(self, value):
        if isValidBaseURL(value) and value.endswith('/') and \
            not value.endswith(':/'):
            return

        raise interfaces.InvalidBaseURL, value

    def fromUnicode(self, value):
        v = str(value.strip())
        self.validate(v)
        return v



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Date Selection Field interfaces

$Id: interfaces.py 91149 2008-09-14 22:39:34Z rogerineichen $
"""
__docformat__ = "reStructuredText"

import zope.schema.interfaces


class IDateSelect(zope.schema.interfaces.IDate):

    yearRange = zope.interface.Attribute(u"Year range.")

    initialDate = zope.interface.Attribute(
        u"Initial date displayed or ``None``. If ``None``, `today()`` is used.")



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: field.py 71296 2006-11-24 03:30:10Z rogerineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.schema

from z3c.schema.email import interfaces

rfc822_specials = '()<>@,;:\\"[]'


def isValidMailAddress(addr):
    """Returns True if the email address is valid and False if not."""

    # First we validate the name portion (name@domain)
    c = 0
    while c < len(addr):
        if addr[c] == '@':
            break
        # Make sure there are only ASCII characters
        if ord(addr[c]) <= 32 or ord(addr[c]) >= 127:
            return False
        # A RFC-822 address cannot contain certain ASCII characters
        if addr[c] in rfc822_specials:
            return False
        c = c + 1

    # check whether we have any input and that the name did not end with a dot
    if not c or addr[c - 1] == '.':
        return False

    # check also starting and ending dots in (name@domain)
    if addr.startswith('.') or addr.endswith('.'):
        return False

    # Next we validate the domain portion (name@domain)
    domain = c = c + 1
    # Ensure that the domain is not empty (name@)
    if domain >= len(addr):
        return False
    count = 0
    while c < len(addr):
        # Make sure that domain does not end with a dot or has two dots in a row
        if addr[c] == '.':
            if c == domain or addr[c - 1] == '.':
                return False
            count = count + 1
        # Make sure there are only ASCII characters
        if ord(addr[c]) <= 32 or ord(addr[c]) >= 127:
            return False
        # A RFC-822 address cannot contain certain ASCII characters
        if addr[c] in rfc822_specials:
            return False
        c = c + 1
    if count >= 1:
        return True
    else:
        return False


class RFC822MailAddress(zope.schema.TextLine):
    """A valid email address."""
    __doc__ = interfaces.IRFC822MailAddress.__doc__

    zope.interface.implements(interfaces.IRFC822MailAddress)

    def constraint(self, value):
        return '\n' not in value and '\r' not in value

    def _validate(self, value):
        super(RFC822MailAddress, self)._validate(value)
        if not isValidMailAddress(value):
            raise interfaces.NotValidRFC822MailAdress(value)



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from zope.schema.interfaces import RequiredMissing
from zope.schema.tests.test_field import FieldTestBase
import doctest
import unittest

from z3c.schema.baseurl import BaseURL
from z3c.schema.baseurl import InvalidBaseURL


class BaseURLTest(FieldTestBase):

    _Field_Factory = BaseURL
    _convert = str

    def testValidate(self):
        field = self._Field_Factory(title=u'BaseURL field', description=u'',
            readonly=False, required=False)
        field.validate(None)
        field.validate(self._convert('host:123.123.123.123/'))
        field.validate(self._convert('host:123/'))
        field.validate(self._convert('http://www.host.com:123/'))
        field.validate(self._convert('http://123.123.123.123:123/'))

    def testValidateRequired(self):
        field = self._Field_Factory(title=u'BaseURL field', description=u'',
            readonly=False, required=True)
        self.assertRaises(RequiredMissing, field.validate, None)

    def testBadStringType(self):
        field = self._Field_Factory(title=u'BaseURL field')
        self.assertRaises(InvalidBaseURL, field.validate, u'123.123')

    def test_newlines(self):
        field = self._Field_Factory(title=u'BaseURL field')
        self.assertRaises(InvalidBaseURL, field.validate,
            self._convert('host\nfoo'))


def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('README.txt',
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,),
        unittest.makeSuite(BaseURLTest),
        ))



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: field.py 91149 2008-09-14 22:39:34Z rogerineichen $
"""
__docformat__ = "reStructuredText"

import re

import zope.interface
import zope.schema

from z3c.schema.hostname import interfaces


isValidHostName = re.compile(

    # BEGIN: host
    r"([a-zA-Z]+([a-zA-Z\d\-]*[a-zA-Z\d])*"
    r"(\.[a-zA-Z\d]+([a-zA-Z\d\-]*[a-zA-Z\d])*)*"
    # END: host

    # or
    r"|"

    # BEGIN: IP
    r"([1-9][\d]{0,1}|1[\d]{0,2}|2[0-5]{0,2})"
    r"(\.([\d]{1,2}|1[\d]{0,2}|2[0-5]{0,2})){3})"
    # END: IP

    # port
    r"(:[\d]{1,5})?$"
    ).match


class HostName(zope.schema.URI):
    """HostName schema field.

    This is a IP Address or a host name.
    """

    zope.interface.implements(interfaces.IHostName)

    def _validate(self, value):
        if isValidHostName(value):
            return
        raise interfaces.InvalidHostName, value

    def fromUnicode(self, value):
        v = str(value.strip())
        self.validate(v)
        return v



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Date Selection field implementation

$Id: field.py 91149 2008-09-14 22:39:34Z rogerineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.schema

from z3c.schema.dateselect import interfaces


class DateSelect(zope.schema.Date):
    zope.interface.implements(interfaces.IDateSelect)

    yearRange = range(1900, 2100)
    initialDate = None # set a date or today is used

    def __init__(self, yearRange=None, initialDate=None, **kw):
        super(DateSelect, self).__init__(**kw)
        self.initialDate = initialDate
        if yearRange is not None:
            self.yearRange = yearRange



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup

$Id: setup.py 121242 2011-04-03 20:17:56Z janjaapdriessen $
"""
import os
import xml.sax.saxutils
from setuptools import setup, find_packages

def read(*rnames):
    text = open(os.path.join(os.path.dirname(__file__), *rnames)).read()
    return text + '\n\n'

setup(
    name = 'z3c.schema',
    version='0.7.2',
    author = 'Zope Community',
      author_email = "zope-dev@zope.org",
      description = "Additional schema fields for Zope 3",
    long_description=(
        read('README.txt') +
        '.. contents::\n\n'+
        read('CHANGES.txt') +
        read('src', 'z3c', 'schema', 'baseurl', 'README.txt') +
        read('src', 'z3c', 'schema', 'dateselect', 'README.txt') +
        read('src', 'z3c', 'schema', 'email', 'README.txt') +
        read('src', 'z3c', 'schema', 'hostname', 'README.txt') +
        read('src', 'z3c', 'schema', 'ip', 'README.txt') +
        read('src', 'z3c', 'schema', 'optchoice', 'README.txt') +
        read('src', 'z3c', 'schema', 'payments', 'README.txt') +
        read('src', 'z3c', 'schema', 'regex', 'README.txt')
        ),
    license = 'ZPL 2.1',
    keywords = 'zope zope3 z3c schema',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Natural Language :: English',
        'Operating System :: Microsoft :: Windows',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope3'],
    url = 'http://pypi.python.org/pypi/z3c.schema',
    packages = find_packages('src'),
    include_package_data=True,
    package_dir = {'':'src'},
    namespace_packages=['z3c',],
    extras_require = dict(
      test=[
          'zope.testing',
           ]
    ),
    install_requires = [
      'setuptools',
      'zope.i18nmessageid',
      'zope.interface',
      'zope.schema >= 3.6',
      ],
    zip_safe=False,
)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: field.py 91149 2008-09-14 22:39:34Z rogerineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.schema

from z3c.schema.ip import interfaces

def isValidIPAddress(addr):
    """Returns True if the IP address is valid and False if not."""
    # Check that we have four pieces separated by .
    pieces = addr.split('.')
    if len(pieces) != 4:
        return False
    # Now check that each piece is an integer between 0 and 255
    for piece in pieces:
        try:
            num = int(piece)
        except ValueError:
            return False
        if not (num >= 0 and num <= 255):
            return False
    return True


class IPAddress(zope.schema.BytesLine):
    """A valid IP address."""
    zope.interface.implements(interfaces.IIPAddress)

    def _validate(self, value):
        super(IPAddress, self)._validate(value)

        if not isValidIPAddress(value):
            raise interfaces.NotValidIPAdress(value)



###############################################################################
#
# Copyright 2006 by refline (Schweiz) AG, CH-5630 Muri
#
###############################################################################
"""Refline Recruiter Tests

$Id: tests.py 117620 2010-10-18 08:00:36Z icemac $
"""
__docformat__ = 'restructuredtext'

from zope.schema.interfaces import RequiredMissing
from zope.schema.tests.test_field import FieldTestBase
import doctest
import unittest

from z3c.schema.hostname import HostName
from z3c.schema.hostname import InvalidHostName


class HostNameTest(FieldTestBase):

    _Field_Factory = HostName
    _convert = str

    def testValidate(self):
        field = self._Field_Factory(title=u'HostName field', description=u'',
            readonly=False, required=False)
        field.validate(None)
        field.validate(self._convert('host'))
        field.validate(self._convert('123.123.123.123'))
        field.validate(self._convert('host:123'))
        field.validate(self._convert('www.host.com:123'))
        field.validate(self._convert('123.123.123.123:123'))

    def testValidateRequired(self):
        field = self._Field_Factory(title=u'HostName field', description=u'',
            readonly=False, required=True)
        field.validate(self._convert('host'))
        self.assertRaises(RequiredMissing, field.validate, None)

    def testBadStringType(self):
        field = self._Field_Factory(title=u'HostName field')
        self.assertRaises(InvalidHostName, field.validate, u'123.123')

    def test_newlines(self):
        field = self._Field_Factory(title=u'HostName field')
        self.assertRaises(InvalidHostName, field.validate,
            self._convert('host\nfoo'))


def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('README.txt',
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,),
        unittest.makeSuite(HostNameTest),
        ))
